@@ -2,9 +2,13 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+(function(global, shared, exports) {
+
 'use strict';
 
-var $RegExp = global.RegExp;
+%CheckIsBootstrapping();
+
+var GlobalRegExp = global.RegExp;
 
 // -------------------------------------------------------------------
 
@@ -12,8 +16,7 @@ var $RegExp = global.RegExp;
 // + https://bugs.ecmascript.org/show_bug.cgi?id=3423
 function RegExpGetFlags() {
   if (!IS_SPEC_OBJECT(this)) {
-    throw MakeTypeError('flags_getter_non_object',
-                        [%ToString(this)]);
+    throw MakeTypeError(kFlagsGetterNonObject, $toString(this));
   }
   var result = '';
   if (this.global) result += 'g';
@@ -24,12 +27,8 @@ function RegExpGetFlags() {
   return result;
 }
 
-function ExtendRegExpPrototype() {
-  %CheckIsBootstrapping();
+%DefineAccessorPropertyUnchecked(GlobalRegExp.prototype, 'flags',
+                                 RegExpGetFlags, null, DONT_ENUM);
+%SetNativeFlag(RegExpGetFlags);
 
-  %DefineAccessorPropertyUnchecked($RegExp.prototype, 'flags', RegExpGetFlags,
-                                   null, DONT_ENUM);
-  %SetNativeFlag(RegExpGetFlags);
-}
-
-ExtendRegExpPrototype();
+})
