@@ -4,6 +4,7 @@
 
 #include "src/v8.h"
 
+#include "src/cpu-profiler.h"
 #include "src/ic/call-optimization.h"
 #include "src/ic/handler-compiler.h"
 #include "src/ic/ic.h"
@@ -73,7 +74,7 @@ Handle<Code> PropertyHandlerCompiler::GetCode(Code::Kind kind,
                                               Handle<Name> name) {
   Code::Flags flags = Code::ComputeHandlerFlags(kind, type, cache_holder());
   Handle<Code> code = GetCodeWithFlags(flags, name);
-  PROFILE(isolate(), CodeCreateEvent(Logger::STUB_TAG, *code, *name));
+  PROFILE(isolate(), CodeCreateEvent(Logger::HANDLER_TAG, *code, *name));
 #ifdef DEBUG
   code->VerifyEmbeddedObjects();
 #endif
@@ -86,7 +87,8 @@ Handle<Code> PropertyHandlerCompiler::GetCode(Code::Kind kind,
 
 Register NamedLoadHandlerCompiler::FrontendHeader(Register object_reg,
                                                   Handle<Name> name,
-                                                  Label* miss) {
+                                                  Label* miss,
+                                                  ReturnHolder return_what) {
   PrototypeCheckType check_type = CHECK_ALL_MAPS;
   int function_index = -1;
   if (map()->instance_type() < FIRST_NONSTRING_TYPE) {
@@ -113,7 +115,7 @@ Register NamedLoadHandlerCompiler::FrontendHeader(Register object_reg,
 
   // Check that the maps starting from the prototype haven't changed.
   return CheckPrototypes(object_reg, scratch1(), scratch2(), scratch3(), name,
-                         miss, check_type);
+                         miss, check_type, return_what);
 }
 
 
@@ -121,9 +123,10 @@ Register NamedLoadHandlerCompiler::FrontendHeader(Register object_reg,
 // miss.
 Register NamedStoreHandlerCompiler::FrontendHeader(Register object_reg,
                                                    Handle<Name> name,
-                                                   Label* miss) {
+                                                   Label* miss,
+                                                   ReturnHolder return_what) {
   return CheckPrototypes(object_reg, this->name(), scratch1(), scratch2(), name,
-                         miss, SKIP_RECEIVER);
+                         miss, SKIP_RECEIVER, return_what);
 }
 
 
@@ -132,7 +135,7 @@ Register PropertyHandlerCompiler::Frontend(Handle<Name> name) {
   if (IC::ICUseVector(kind())) {
     PushVectorAndSlot();
   }
-  Register reg = FrontendHeader(receiver(), name, &miss);
+  Register reg = FrontendHeader(receiver(), name, &miss, RETURN_HOLDER);
   FrontendFooter(name, &miss);
   // The footer consumes the vector and slot from the stack if miss occurs.
   if (IC::ICUseVector(kind())) {
@@ -155,8 +158,13 @@ void PropertyHandlerCompiler::NonexistentFrontendHeader(Handle<Name> name,
     // Handle<JSObject>::null().
     DCHECK(last_map->prototype() == isolate()->heap()->null_value());
   } else {
-    holder_reg = FrontendHeader(receiver(), name, miss);
     last_map = handle(holder()->map());
+    // This condition matches the branches below.
+    bool need_holder =
+        last_map->is_dictionary_map() && !last_map->IsJSGlobalObjectMap();
+    holder_reg =
+        FrontendHeader(receiver(), name, miss,
+                       need_holder ? RETURN_HOLDER : DONT_RETURN_ANYTHING);
   }
 
   if (last_map->is_dictionary_map()) {
@@ -279,6 +287,7 @@ Handle<Code> NamedLoadHandlerCompiler::CompileLoadInterceptor(
     case LookupIterator::INTERCEPTOR:
     case LookupIterator::JSPROXY:
     case LookupIterator::NOT_FOUND:
+    case LookupIterator::INTEGER_INDEXED_EXOTIC:
       break;
     case LookupIterator::DATA:
       inline_followup =
@@ -310,10 +319,36 @@ Handle<Code> NamedLoadHandlerCompiler::CompileLoadInterceptor(
 
   Label miss;
   InterceptorVectorSlotPush(receiver());
-  Register reg = FrontendHeader(receiver(), it->name(), &miss);
+  bool lost_holder_register = false;
+  auto holder_orig = holder();
+  // non masking interceptors must check the entire chain, so temporarily reset
+  // the holder to be that last element for the FrontendHeader call.
+  if (holder()->GetNamedInterceptor()->non_masking()) {
+    DCHECK(!inline_followup);
+    JSObject* last = *holder();
+    PrototypeIterator iter(isolate(), last);
+    while (!iter.IsAtEnd()) {
+      lost_holder_register = true;
+      last = JSObject::cast(iter.GetCurrent());
+      iter.Advance();
+    }
+    auto last_handle = handle(last);
+    set_holder(last_handle);
+  }
+  Register reg = FrontendHeader(receiver(), it->name(), &miss, RETURN_HOLDER);
+  // Reset the holder so further calculations are correct.
+  set_holder(holder_orig);
+  if (lost_holder_register) {
+    if (*it->GetReceiver() == *holder()) {
+      reg = receiver();
+    } else {
+      // Reload lost holder register.
+      auto cell = isolate()->factory()->NewWeakCell(holder());
+      __ LoadWeakValue(reg, cell, &miss);
+    }
+  }
   FrontendFooter(it->name(), &miss);
   InterceptorVectorSlotPop(reg);
-
   if (inline_followup) {
     // TODO(368): Compile in the whole chain: all the interceptors in
     // prototypes and ultimate answer.
@@ -335,7 +370,8 @@ void NamedLoadHandlerCompiler::GenerateLoadPostInterceptor(
 
   Label miss;
   InterceptorVectorSlotPush(interceptor_reg);
-  Register reg = FrontendHeader(interceptor_reg, it->name(), &miss);
+  Register reg =
+      FrontendHeader(interceptor_reg, it->name(), &miss, RETURN_HOLDER);
   FrontendFooter(it->name(), &miss);
   // We discard the vector and slot now because we don't miss below this point.
   InterceptorVectorSlotPop(reg, DISCARD);
@@ -345,6 +381,7 @@ void NamedLoadHandlerCompiler::GenerateLoadPostInterceptor(
     case LookupIterator::INTERCEPTOR:
     case LookupIterator::JSPROXY:
     case LookupIterator::NOT_FOUND:
+    case LookupIterator::INTEGER_INDEXED_EXOTIC:
     case LookupIterator::TRANSITION:
       UNREACHABLE();
     case LookupIterator::DATA: {
@@ -399,7 +436,7 @@ Handle<Code> NamedStoreHandlerCompiler::CompileStoreTransition(
     if (!last.is_null()) set_holder(last);
     NonexistentFrontendHeader(name, &miss, scratch1(), scratch2());
   } else {
-    FrontendHeader(receiver(), name, &miss);
+    FrontendHeader(receiver(), name, &miss, DONT_RETURN_ANYTHING);
     DCHECK(holder()->HasFastProperties());
   }
 
@@ -499,6 +536,13 @@ void ElementHandlerCompiler::CompileElementHandlers(
     } else {
       bool is_js_array = receiver_map->instance_type() == JS_ARRAY_TYPE;
       ElementsKind elements_kind = receiver_map->elements_kind();
+
+      // No need to check for an elements-free prototype chain here, the
+      // generated stub code needs to check that dynamically anyway.
+      bool convert_hole_to_undefined =
+          is_js_array && elements_kind == FAST_HOLEY_ELEMENTS &&
+          *receiver_map == isolate()->get_initial_js_array_map(elements_kind);
+
       if (receiver_map->has_indexed_interceptor()) {
         cached_stub = LoadIndexedInterceptorStub(isolate()).GetCode();
       } else if (IsSloppyArgumentsElements(elements_kind)) {
@@ -506,8 +550,8 @@ void ElementHandlerCompiler::CompileElementHandlers(
       } else if (IsFastElementsKind(elements_kind) ||
                  IsExternalArrayElementsKind(elements_kind) ||
                  IsFixedTypedArrayElementsKind(elements_kind)) {
-        cached_stub = LoadFastElementStub(isolate(), is_js_array, elements_kind)
-                          .GetCode();
+        cached_stub = LoadFastElementStub(isolate(), is_js_array, elements_kind,
+                                          convert_hole_to_undefined).GetCode();
       } else {
         DCHECK(elements_kind == DICTIONARY_ELEMENTS);
         cached_stub = LoadDictionaryElementStub(isolate()).GetCode();
