@@ -2,8 +2,25 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+var $observeNotifyChange;
+var $observeEnqueueSpliceRecord;
+var $observeBeginPerformSplice;
+var $observeEndPerformSplice;
+var $observeNativeObjectObserve;
+var $observeNativeObjectGetNotifier;
+var $observeNativeObjectNotifierPerformChange;
+
+(function(global, shared, exports) {
+
 "use strict";
 
+%CheckIsBootstrapping();
+
+var GlobalArray = global.Array;
+var GlobalObject = global.Object;
+
+// -------------------------------------------------------------------
+
 // Overview:
 //
 // This file contains all of the routing and accounting for Object.observe.
@@ -35,6 +52,8 @@
 
 var observationState;
 
+var notifierPrototype = {};
+
 // We have to wait until after bootstrapping to grab a reference to the
 // observationState object, since it's not possible to serialize that
 // reference into the snapshot.
@@ -56,34 +75,42 @@ function GetObservationStateJS() {
   return observationState;
 }
 
+
 function GetPendingObservers() {
   return GetObservationStateJS().pendingObservers;
 }
 
+
 function SetPendingObservers(pendingObservers) {
   GetObservationStateJS().pendingObservers = pendingObservers;
 }
 
+
 function GetNextCallbackPriority() {
   return GetObservationStateJS().nextCallbackPriority++;
 }
 
+
 function nullProtoObject() {
   return { __proto__: null };
 }
 
+
 function TypeMapCreate() {
   return nullProtoObject();
 }
 
+
 function TypeMapAddType(typeMap, type, ignoreDuplicate) {
   typeMap[type] = ignoreDuplicate ? 1 : (typeMap[type] || 0) + 1;
 }
 
+
 function TypeMapRemoveType(typeMap, type) {
   typeMap[type]--;
 }
 
+
 function TypeMapCreateFromList(typeList, length) {
   var typeMap = TypeMapCreate();
   for (var i = 0; i < length; i++) {
@@ -92,10 +119,12 @@ function TypeMapCreateFromList(typeList, length) {
   return typeMap;
 }
 
+
 function TypeMapHasType(typeMap, type) {
   return !!typeMap[type];
 }
 
+
 function TypeMapIsDisjointFrom(typeMap1, typeMap2) {
   if (!typeMap1 || !typeMap2)
     return true;
@@ -108,6 +137,7 @@ function TypeMapIsDisjointFrom(typeMap1, typeMap2) {
   return true;
 }
 
+
 var defaultAcceptTypes = (function() {
   var defaultTypes = [
     'add',
@@ -120,6 +150,7 @@ var defaultAcceptTypes = (function() {
   return TypeMapCreateFromList(defaultTypes, defaultTypes.length);
 })();
 
+
 // An Observer is a registration to observe an object by a callback with
 // a given set of accept types. If the set of accept types is the default
 // set for Object.observe, the observer is represented as a direct reference
@@ -134,19 +165,23 @@ function ObserverCreate(callback, acceptList) {
   return observer;
 }
 
+
 function ObserverGetCallback(observer) {
   return IS_SPEC_FUNCTION(observer) ? observer : observer.callback;
 }
 
+
 function ObserverGetAcceptTypes(observer) {
   return IS_SPEC_FUNCTION(observer) ? defaultAcceptTypes : observer.accept;
 }
 
+
 function ObserverIsActive(observer, objectInfo) {
   return TypeMapIsDisjointFrom(ObjectInfoGetPerformingTypes(objectInfo),
                                ObserverGetAcceptTypes(observer));
 }
 
+
 function ObjectInfoGetOrCreate(object) {
   var objectInfo = ObjectInfoGet(object);
   if (IS_UNDEFINED(objectInfo)) {
@@ -166,15 +201,18 @@ function ObjectInfoGetOrCreate(object) {
   return objectInfo;
 }
 
+
 function ObjectInfoGet(object) {
   return %WeakCollectionGet(GetObservationStateJS().objectInfoMap, object);
 }
 
+
 function ObjectInfoGetFromNotifier(notifier) {
   return %WeakCollectionGet(GetObservationStateJS().notifierObjectInfoMap,
                             notifier);
 }
 
+
 function ObjectInfoGetNotifier(objectInfo) {
   if (IS_NULL(objectInfo.notifier)) {
     objectInfo.notifier = { __proto__: notifierPrototype };
@@ -185,11 +223,13 @@ function ObjectInfoGetNotifier(objectInfo) {
   return objectInfo.notifier;
 }
 
+
 function ChangeObserversIsOptimized(changeObservers) {
   return IS_SPEC_FUNCTION(changeObservers) ||
          IS_SPEC_FUNCTION(changeObservers.callback);
 }
 
+
 // The set of observers on an object is called 'changeObservers'. The first
 // observer is referenced directly via objectInfo.changeObservers. When a second
 // is added, changeObservers "normalizes" to become a mapping of callback
@@ -205,6 +245,7 @@ function ObjectInfoNormalizeChangeObservers(objectInfo) {
   }
 }
 
+
 function ObjectInfoAddObserver(objectInfo, callback, acceptList) {
   var callbackInfo = CallbackInfoGetOrCreate(callback);
   var observer = ObserverCreate(callback, acceptList);
@@ -250,35 +291,39 @@ function ObjectInfoHasActiveObservers(objectInfo) {
   return false;
 }
 
+
 function ObjectInfoAddPerformingType(objectInfo, type) {
   objectInfo.performing = objectInfo.performing || TypeMapCreate();
   TypeMapAddType(objectInfo.performing, type);
   objectInfo.performingCount++;
 }
 
+
 function ObjectInfoRemovePerformingType(objectInfo, type) {
   objectInfo.performingCount--;
   TypeMapRemoveType(objectInfo.performing, type);
 }
 
+
 function ObjectInfoGetPerformingTypes(objectInfo) {
   return objectInfo.performingCount > 0 ? objectInfo.performing : null;
 }
 
+
 function ConvertAcceptListToTypeMap(arg) {
   // We use undefined as a sentinel for the default accept list.
   if (IS_UNDEFINED(arg))
     return arg;
 
-  if (!IS_SPEC_OBJECT(arg))
-    throw MakeTypeError("observe_accept_invalid");
+  if (!IS_SPEC_OBJECT(arg)) throw MakeTypeError(kObserveInvalidAccept);
 
-  var len = ToInteger(arg.length);
+  var len = $toInteger(arg.length);
   if (len < 0) len = 0;
 
   return TypeMapCreateFromList(arg, len);
 }
 
+
 // CallbackInfo's optimized state is just a number which represents its global
 // priority. When a change record must be enqueued for the callback, it
 // normalizes. When delivery clears any pending change records, it re-optimizes.
@@ -286,11 +331,13 @@ function CallbackInfoGet(callback) {
   return %WeakCollectionGet(GetObservationStateJS().callbackInfoMap, callback);
 }
 
+
 function CallbackInfoSet(callback, callbackInfo) {
   %WeakCollectionSet(GetObservationStateJS().callbackInfoMap,
                      callback, callbackInfo);
 }
 
+
 function CallbackInfoGetOrCreate(callback) {
   var callbackInfo = CallbackInfoGet(callback);
   if (!IS_UNDEFINED(callbackInfo))
@@ -301,6 +348,7 @@ function CallbackInfoGetOrCreate(callback) {
   return priority;
 }
 
+
 function CallbackInfoGetPriority(callbackInfo) {
   if (IS_NUMBER(callbackInfo))
     return callbackInfo;
@@ -308,6 +356,7 @@ function CallbackInfoGetPriority(callbackInfo) {
     return callbackInfo.priority;
 }
 
+
 function CallbackInfoNormalize(callback) {
   var callbackInfo = CallbackInfoGet(callback);
   if (IS_NUMBER(callbackInfo)) {
@@ -319,20 +368,22 @@ function CallbackInfoNormalize(callback) {
   return callbackInfo;
 }
 
+
 function ObjectObserve(object, callback, acceptList) {
   if (!IS_SPEC_OBJECT(object))
-    throw MakeTypeError("observe_non_object", ["observe"]);
+    throw MakeTypeError(kObserveNonObject, "observe", "observe");
   if (%IsJSGlobalProxy(object))
-    throw MakeTypeError("observe_global_proxy", ["observe"]);
+    throw MakeTypeError(kObserveGlobalProxy, "observe");
   if (!IS_SPEC_FUNCTION(callback))
-    throw MakeTypeError("observe_non_function", ["observe"]);
-  if (ObjectIsFrozen(callback))
-    throw MakeTypeError("observe_callback_frozen");
+    throw MakeTypeError(kObserveNonFunction, "observe");
+  if ($objectIsFrozen(callback))
+    throw MakeTypeError(kObserveCallbackFrozen);
 
   var objectObserveFn = %GetObjectContextObjectObserve(object);
   return objectObserveFn(object, callback, acceptList);
 }
 
+
 function NativeObjectObserve(object, callback, acceptList) {
   var objectInfo = ObjectInfoGetOrCreate(object);
   var typeList = ConvertAcceptListToTypeMap(acceptList);
@@ -340,13 +391,14 @@ function NativeObjectObserve(object, callback, acceptList) {
   return object;
 }
 
+
 function ObjectUnobserve(object, callback) {
   if (!IS_SPEC_OBJECT(object))
-    throw MakeTypeError("observe_non_object", ["unobserve"]);
+    throw MakeTypeError(kObserveNonObject, "unobserve", "unobserve");
   if (%IsJSGlobalProxy(object))
-    throw MakeTypeError("observe_global_proxy", ["unobserve"]);
+    throw MakeTypeError(kObserveGlobalProxy, "unobserve");
   if (!IS_SPEC_FUNCTION(callback))
-    throw MakeTypeError("observe_non_function", ["unobserve"]);
+    throw MakeTypeError(kObserveNonFunction, "unobserve");
 
   var objectInfo = ObjectInfoGet(object);
   if (IS_UNDEFINED(objectInfo))
@@ -356,6 +408,7 @@ function ObjectUnobserve(object, callback) {
   return object;
 }
 
+
 function ArrayObserve(object, callback) {
   return ObjectObserve(object, callback, ['add',
                                           'update',
@@ -363,10 +416,12 @@ function ArrayObserve(object, callback) {
                                           'splice']);
 }
 
+
 function ArrayUnobserve(object, callback) {
   return ObjectUnobserve(object, callback);
 }
 
+
 function ObserverEnqueueIfActive(observer, objectInfo, changeRecord) {
   if (!ObserverIsActive(observer, objectInfo) ||
       !TypeMapHasType(ObserverGetAcceptTypes(observer), changeRecord.type)) {
@@ -399,6 +454,7 @@ function ObserverEnqueueIfActive(observer, objectInfo, changeRecord) {
   callbackInfo.push(changeRecord);
 }
 
+
 function ObjectInfoEnqueueExternalChangeRecord(objectInfo, changeRecord, type) {
   if (!ObjectInfoHasActiveObservers(objectInfo))
     return;
@@ -413,11 +469,12 @@ function ObjectInfoEnqueueExternalChangeRecord(objectInfo, changeRecord, type) {
     %DefineDataPropertyUnchecked(
         newRecord, prop, changeRecord[prop], READ_ONLY + DONT_DELETE);
   }
-  ObjectFreezeJS(newRecord);
+  $objectFreeze(newRecord);
 
   ObjectInfoEnqueueInternalChangeRecord(objectInfo, newRecord);
 }
 
+
 function ObjectInfoEnqueueInternalChangeRecord(objectInfo, changeRecord) {
   // TODO(rossberg): adjust once there is a story for symbols vs proxies.
   if (IS_SYMBOL(changeRecord.name)) return;
@@ -436,18 +493,21 @@ function ObjectInfoEnqueueInternalChangeRecord(objectInfo, changeRecord) {
   }
 }
 
+
 function BeginPerformSplice(array) {
   var objectInfo = ObjectInfoGet(array);
   if (!IS_UNDEFINED(objectInfo))
     ObjectInfoAddPerformingType(objectInfo, 'splice');
 }
 
+
 function EndPerformSplice(array) {
   var objectInfo = ObjectInfoGet(array);
   if (!IS_UNDEFINED(objectInfo))
     ObjectInfoRemovePerformingType(objectInfo, 'splice');
 }
 
+
 function EnqueueSpliceRecord(array, index, removed, addedCount) {
   var objectInfo = ObjectInfoGet(array);
   if (!ObjectInfoHasActiveObservers(objectInfo))
@@ -461,11 +521,12 @@ function EnqueueSpliceRecord(array, index, removed, addedCount) {
     addedCount: addedCount
   };
 
-  ObjectFreezeJS(changeRecord);
-  ObjectFreezeJS(changeRecord.removed);
+  $objectFreeze(changeRecord);
+  $objectFreeze(changeRecord.removed);
   ObjectInfoEnqueueInternalChangeRecord(objectInfo, changeRecord);
 }
 
+
 function NotifyChange(type, object, name, oldValue) {
   var objectInfo = ObjectInfoGet(object);
   if (!ObjectInfoHasActiveObservers(objectInfo))
@@ -485,41 +546,42 @@ function NotifyChange(type, object, name, oldValue) {
     };
   }
 
-  ObjectFreezeJS(changeRecord);
+  $objectFreeze(changeRecord);
   ObjectInfoEnqueueInternalChangeRecord(objectInfo, changeRecord);
 }
 
-var notifierPrototype = {};
 
 function ObjectNotifierNotify(changeRecord) {
   if (!IS_SPEC_OBJECT(this))
-    throw MakeTypeError("called_on_non_object", ["notify"]);
+    throw MakeTypeError(kCalledOnNonObject, "notify");
 
   var objectInfo = ObjectInfoGetFromNotifier(this);
   if (IS_UNDEFINED(objectInfo))
-    throw MakeTypeError("observe_notify_non_notifier");
+    throw MakeTypeError(kObserveNotifyNonNotifier);
   if (!IS_STRING(changeRecord.type))
-    throw MakeTypeError("observe_type_non_string");
+    throw MakeTypeError(kObserveTypeNonString);
 
   ObjectInfoEnqueueExternalChangeRecord(objectInfo, changeRecord);
 }
 
+
 function ObjectNotifierPerformChange(changeType, changeFn) {
   if (!IS_SPEC_OBJECT(this))
-    throw MakeTypeError("called_on_non_object", ["performChange"]);
+    throw MakeTypeError(kCalledOnNonObject, "performChange");
 
   var objectInfo = ObjectInfoGetFromNotifier(this);
   if (IS_UNDEFINED(objectInfo))
-    throw MakeTypeError("observe_notify_non_notifier");
+    throw MakeTypeError(kObserveNotifyNonNotifier);
   if (!IS_STRING(changeType))
-    throw MakeTypeError("observe_perform_non_string");
+    throw MakeTypeError(kObservePerformNonString);
   if (!IS_SPEC_FUNCTION(changeFn))
-    throw MakeTypeError("observe_perform_non_function");
+    throw MakeTypeError(kObservePerformNonFunction);
 
   var performChangeFn = %GetObjectContextNotifierPerformChange(objectInfo);
   performChangeFn(objectInfo, changeType, changeFn);
 }
 
+
 function NativeObjectNotifierPerformChange(objectInfo, changeType, changeFn) {
   ObjectInfoAddPerformingType(objectInfo, changeType);
 
@@ -534,13 +596,14 @@ function NativeObjectNotifierPerformChange(objectInfo, changeType, changeFn) {
     ObjectInfoEnqueueExternalChangeRecord(objectInfo, changeRecord, changeType);
 }
 
+
 function ObjectGetNotifier(object) {
   if (!IS_SPEC_OBJECT(object))
-    throw MakeTypeError("observe_non_object", ["getNotifier"]);
+    throw MakeTypeError(kObserveNonObject, "getNotifier", "getNotifier");
   if (%IsJSGlobalProxy(object))
-    throw MakeTypeError("observe_global_proxy", ["getNotifier"]);
+    throw MakeTypeError(kObserveGlobalProxy, "getNotifier");
 
-  if (ObjectIsFrozen(object)) return null;
+  if ($objectIsFrozen(object)) return null;
 
   if (!%ObjectWasCreatedInCurrentOrigin(object)) return null;
 
@@ -548,11 +611,13 @@ function ObjectGetNotifier(object) {
   return getNotifierFn(object);
 }
 
+
 function NativeObjectGetNotifier(object) {
   var objectInfo = ObjectInfoGetOrCreate(object);
   return ObjectInfoGetNotifier(objectInfo);
 }
 
+
 function CallbackDeliverPending(callback) {
   var callbackInfo = CallbackInfoGet(callback);
   if (IS_UNDEFINED(callbackInfo) || IS_NUMBER(callbackInfo))
@@ -575,13 +640,15 @@ function CallbackDeliverPending(callback) {
   return true;
 }
 
+
 function ObjectDeliverChangeRecords(callback) {
   if (!IS_SPEC_FUNCTION(callback))
-    throw MakeTypeError("observe_non_function", ["deliverChangeRecords"]);
+    throw MakeTypeError(kObserveNonFunction, "deliverChangeRecords");
 
   while (CallbackDeliverPending(callback)) {}
 }
 
+
 function ObserveMicrotaskRunner() {
   var pendingObservers = GetPendingObservers();
   if (!IS_NULL(pendingObservers)) {
@@ -592,22 +659,29 @@ function ObserveMicrotaskRunner() {
   }
 }
 
-function SetupObjectObserve() {
-  %CheckIsBootstrapping();
-  InstallFunctions($Object, DONT_ENUM, $Array(
-    "deliverChangeRecords", ObjectDeliverChangeRecords,
-    "getNotifier", ObjectGetNotifier,
-    "observe", ObjectObserve,
-    "unobserve", ObjectUnobserve
-  ));
-  InstallFunctions($Array, DONT_ENUM, $Array(
-    "observe", ArrayObserve,
-    "unobserve", ArrayUnobserve
-  ));
-  InstallFunctions(notifierPrototype, DONT_ENUM, $Array(
-    "notify", ObjectNotifierNotify,
-    "performChange", ObjectNotifierPerformChange
-  ));
-}
+// -------------------------------------------------------------------
 
-SetupObjectObserve();
+$installFunctions(GlobalObject, DONT_ENUM, [
+  "deliverChangeRecords", ObjectDeliverChangeRecords,
+  "getNotifier", ObjectGetNotifier,
+  "observe", ObjectObserve,
+  "unobserve", ObjectUnobserve
+]);
+$installFunctions(GlobalArray, DONT_ENUM, [
+  "observe", ArrayObserve,
+  "unobserve", ArrayUnobserve
+]);
+$installFunctions(notifierPrototype, DONT_ENUM, [
+  "notify", ObjectNotifierNotify,
+  "performChange", ObjectNotifierPerformChange
+]);
+
+$observeNotifyChange = NotifyChange;
+$observeEnqueueSpliceRecord = EnqueueSpliceRecord;
+$observeBeginPerformSplice = BeginPerformSplice;
+$observeEndPerformSplice = EndPerformSplice;
+$observeNativeObjectObserve = NativeObjectObserve;
+$observeNativeObjectGetNotifier = NativeObjectGetNotifier;
+$observeNativeObjectNotifierPerformChange = NativeObjectNotifierPerformChange;
+
+})
