@@ -1,16 +1,13 @@
+/* eslint-disable require-buffer */
 'use strict';
 
 const binding = process.binding('buffer');
-const smalloc = process.binding('smalloc');
-const util = require('util');
-const alloc = smalloc.alloc;
-const truncate = smalloc.truncate;
-const sliceOnto = smalloc.sliceOnto;
-const kMaxLength = smalloc.kMaxLength;
+const internalUtil = require('internal/util');
 
 exports.Buffer = Buffer;
 exports.SlowBuffer = SlowBuffer;
 exports.INSPECT_MAX_BYTES = 50;
+exports.kMaxLength = binding.kMaxLength;
 
 
 Buffer.poolSize = 8 * 1024;
@@ -19,188 +16,137 @@ var poolSize, poolOffset, allocPool;
 
 function createPool() {
   poolSize = Buffer.poolSize;
-  allocPool = alloc({}, poolSize);
+  allocPool = binding.create(poolSize);
   poolOffset = 0;
 }
-createPool();
 
-function Buffer(arg) {
-  if (!(this instanceof Buffer)) {
-    // Avoid going through an ArgumentsAdaptorTrampoline in the common case.
-    if (arguments.length > 1)
-      return new Buffer(arg, arguments[1]);
-
-    return new Buffer(arg);
-  }
-
-  this.length = 0;
-  this.parent = undefined;
-
-  // Common case.
-  if (typeof(arg) === 'number') {
-    fromNumber(this, arg);
-    return;
-  }
-
-  // Slightly less common case.
-  if (typeof(arg) === 'string') {
-    fromString(this, arg, arguments.length > 1 ? arguments[1] : 'utf8');
-    return;
-  }
-
-  // Unusual.
-  fromObject(this, arg);
-}
-
-function fromNumber(that, length) {
-  allocate(that, length < 0 ? 0 : checked(length) | 0);
-}
-
-function fromString(that, string, encoding) {
-  if (typeof(encoding) !== 'string' || encoding === '')
-    encoding = 'utf8';
-
-  // Assumption: byteLength() return value is always < kMaxLength.
-  var length = byteLength(string, encoding) | 0;
-  allocate(that, length);
-
-  var actual = that.write(string, encoding) | 0;
-  if (actual !== length) {
-    // Fix up for truncated base64 input.  Don't bother returning
-    // the unused two or three bytes to the pool.
-    that.length = actual;
-    truncate(that, actual);
-  }
-}
-
-function fromObject(that, object) {
-  if (object instanceof Buffer)
-    return fromBuffer(that, object);
-
-  if (Array.isArray(object))
-    return fromArray(that, object);
-
-  if (object == null)
-    throw new TypeError('must start with number, buffer, array or string');
-
-  if (object.buffer instanceof ArrayBuffer)
-    return fromTypedArray(that, object);
-
-  if (object.length)
-    return fromArrayLike(that, object);
-
-  return fromJsonObject(that, object);
-}
-
-function fromBuffer(that, buffer) {
-  var length = checked(buffer.length) | 0;
-  allocate(that, length);
-  buffer.copy(that, 0, 0, length);
-}
-
-function fromArray(that, array) {
-  var length = checked(array.length) | 0;
-  allocate(that, length);
-  for (var i = 0; i < length; i += 1)
-    that[i] = array[i] & 255;
-}
-
-// Duplicate of fromArray() to keep fromArray() monomorphic.
-function fromTypedArray(that, array) {
-  var length = checked(array.length) | 0;
-  allocate(that, length);
-  // Truncating the elements is probably not what people expect from typed
-  // arrays with BYTES_PER_ELEMENT > 1 but it's compatible with the behavior
-  // of the old Buffer constructor.
-  for (var i = 0; i < length; i += 1)
-    that[i] = array[i] & 255;
-}
-
-function fromArrayLike(that, array) {
-  var length = checked(array.length) | 0;
-  allocate(that, length);
-  for (var i = 0; i < length; i += 1)
-    that[i] = array[i] & 255;
-}
-
-// Deserialize { type: 'Buffer', data: [1,2,3,...] } into a Buffer object.
-// Returns a zero-length buffer for inputs that don't conform to the spec.
-function fromJsonObject(that, object) {
-  var array;
-  var length = 0;
-
-  if (object.type === 'Buffer' && Array.isArray(object.data)) {
-    array = object.data;
-    length = checked(array.length) | 0;
-  }
-  allocate(that, length);
-
-  for (var i = 0; i < length; i += 1)
-    that[i] = array[i] & 255;
-}
-
-function allocate(that, length) {
-  var fromPool = length !== 0 && length <= Buffer.poolSize >>> 1;
-  if (fromPool)
-    that.parent = palloc(that, length);
-  else
-    alloc(that, length);
-  that.length = length;
-}
-
-function palloc(that, length) {
-  if (length > poolSize - poolOffset)
-    createPool();
-
-  var start = poolOffset;
-  var end = start + length;
-  var buf = sliceOnto(allocPool, that, start, end);
-  poolOffset = end;
 
+function alignPool() {
   // Ensure aligned slices
   if (poolOffset & 0x7) {
     poolOffset |= 0x7;
     poolOffset++;
   }
-
-  return buf;
 }
 
-function checked(length) {
-  // Note: cannot use `length < kMaxLength` here because that fails when
-  // length is NaN (which is otherwise coerced to zero.)
-  if (length >= kMaxLength) {
-    throw new RangeError('Attempt to allocate Buffer larger than maximum ' +
-                         'size: 0x' + kMaxLength.toString(16) + ' bytes');
+
+function Buffer(arg) {
+  // Common case.
+  if (typeof arg === 'number') {
+    // If less than zero, or NaN.
+    if (arg < 0 || arg !== arg)
+      arg = 0;
+    return allocate(arg);
   }
-  return length >>> 0;
-}
+
+  // Slightly less common case.
+  if (typeof arg === 'string') {
+    return fromString(arg, arguments[1]);
+  }
+
+  // Unusual.
+  return fromObject(arg);
+};
+
+Buffer.prototype.__proto__ = Uint8Array.prototype;
+Buffer.__proto__ = Uint8Array;
+
+
+binding.setupBufferJS(Buffer.prototype);
+// Buffer prototype must be past before creating our first pool.
+createPool();
+
 
 function SlowBuffer(length) {
-  length = length >>> 0;
-  if (length > kMaxLength) {
-    throw new RangeError('Attempt to allocate Buffer larger than maximum ' +
-                         'size: 0x' + kMaxLength.toString(16) + ' bytes');
+  if (length < 0)
+    length = 0;
+  return binding.create(length);
+};
+
+SlowBuffer.prototype.__proto__ = Buffer.prototype;
+SlowBuffer.__proto__ = Buffer;
+
+
+function allocate(size) {
+  if (size === 0)
+    return binding.create(0);
+  if (size < (Buffer.poolSize >>> 1)) {
+    if (size > (poolSize - poolOffset))
+      createPool();
+    var b = binding.slice(allocPool, poolOffset, poolOffset + size);
+    poolOffset += size;
+    alignPool();
+    return b;
+  } else {
+    return binding.create(size);
   }
-  var b = new NativeBuffer(length);
-  alloc(b, length);
+}
+
+
+function fromString(string, encoding) {
+  if (typeof encoding !== 'string' || encoding === '')
+    encoding = 'utf8';
+
+  var length = byteLength(string, encoding);
+  if (length >= (Buffer.poolSize >>> 1))
+    return binding.createFromString(string, encoding);
+
+  if (length > (poolSize - poolOffset))
+    createPool();
+  var actual = allocPool.write(string, poolOffset, encoding);
+  var b = binding.slice(allocPool, poolOffset, poolOffset + actual);
+  poolOffset += actual;
+  alignPool();
   return b;
 }
 
 
-// Bypass all checks for instantiating unallocated Buffer required for
-// Objects created in C++. Significantly faster than calling the Buffer
-// function.
-function NativeBuffer(length) {
-  this.length = length >>> 0;
-  // Set this to keep the object map the same.
-  this.parent = undefined;
+function fromObject(obj) {
+  if (obj instanceof Buffer) {
+    var b = allocate(obj.length);
+    obj.copy(b, 0, 0, obj.length);
+    return b;
+  }
+
+  if (Array.isArray(obj)) {
+    var length = obj.length;
+    var b = allocate(length);
+    for (var i = 0; i < length; i++)
+      b[i] = obj[i] & 255;
+    return b;
+  }
+
+  if (obj == null) {
+    throw new TypeError('must start with number, buffer, array or string');
+  }
+
+  if (obj instanceof ArrayBuffer) {
+    return binding.createFromArrayBuffer(obj);
+  }
+
+  if (obj.buffer instanceof ArrayBuffer || obj.length) {
+    var length;
+    if (typeof obj.length !== 'number' || obj.length !== obj.length)
+      length = 0;
+    else
+      length = obj.length;
+    var b = allocate(length);
+    for (var i = 0; i < length; i++) {
+      b[i] = obj[i] & 255;
+    }
+    return b;
+  }
+
+  if (obj.type === 'Buffer' && Array.isArray(obj.data)) {
+    var array = obj.data;
+    var b = allocate(array.length);
+    for (var i = 0; i < array.length; i++)
+      b[i] = array[i] & 255;
+    return b;
+  }
+
+  throw new TypeError('must start with number, buffer, array or string');
 }
-NativeBuffer.prototype = Buffer.prototype;
-
-
-// add methods to Buffer prototype
-binding.setupBufferJS(NativeBuffer);
 
 
 // Static methods
@@ -212,33 +158,41 @@ Buffer.isBuffer = function isBuffer(b) {
 
 Buffer.compare = function compare(a, b) {
   if (!(a instanceof Buffer) ||
-      !(b instanceof Buffer))
+      !(b instanceof Buffer)) {
     throw new TypeError('Arguments must be Buffers');
+  }
 
-  if (a === b)
+  if (a === b) {
     return 0;
+  }
 
   return binding.compare(a, b);
 };
 
 
 Buffer.isEncoding = function(encoding) {
-  switch ((encoding + '').toLowerCase()) {
-    case 'hex':
-    case 'utf8':
-    case 'utf-8':
-    case 'ascii':
-    case 'binary':
-    case 'base64':
-    case 'ucs2':
-    case 'ucs-2':
-    case 'utf16le':
-    case 'utf-16le':
-    case 'raw':
-      return true;
+  var loweredCase = false;
+  for (;;) {
+    switch (encoding) {
+      case 'hex':
+      case 'utf8':
+      case 'utf-8':
+      case 'ascii':
+      case 'binary':
+      case 'base64':
+      case 'ucs2':
+      case 'ucs-2':
+      case 'utf16le':
+      case 'utf-16le':
+      case 'raw':
+        return true;
 
-    default:
-      return false;
+      default:
+        if (loweredCase)
+          return false;
+        encoding = ('' + encoding).toLowerCase();
+        loweredCase = true;
+    }
   }
 };
 
@@ -249,8 +203,6 @@ Buffer.concat = function(list, length) {
 
   if (list.length === 0)
     return new Buffer(0);
-  else if (list.length === 1)
-    return list[0];
 
   if (length === undefined) {
     length = 0;
@@ -272,36 +224,87 @@ Buffer.concat = function(list, length) {
 };
 
 
-function byteLength(string, encoding) {
-  if (typeof(string) !== 'string')
-    string = String(string);
+function base64ByteLength(str, bytes) {
+  // Handle padding
+  if (str.charCodeAt(bytes - 1) === 0x3D)
+    bytes--;
+  if (bytes > 1 && str.charCodeAt(bytes - 1) === 0x3D)
+    bytes--;
 
-  if (string.length === 0)
+  // Base64 ratio: 3/4
+  return (bytes * 3) >>> 2;
+}
+
+
+function byteLength(string, encoding) {
+  if (typeof string !== 'string')
+    string = '' + string;
+
+  var len = string.length;
+  if (len === 0)
     return 0;
 
-  switch (encoding) {
-    case 'ascii':
-    case 'binary':
-    case 'raw':
-      return string.length;
+  // Use a for loop to avoid recursion
+  var loweredCase = false;
+  for (;;) {
+    switch (encoding) {
+      case 'ascii':
+      case 'binary':
+      // Deprecated
+      case 'raw':
+      case 'raws':
+        return len;
 
-    case 'ucs2':
-    case 'ucs-2':
-    case 'utf16le':
-    case 'utf-16le':
-      return string.length * 2;
+      case 'utf8':
+      case 'utf-8':
+        return binding.byteLengthUtf8(string);
 
-    case 'hex':
-      return string.length >>> 1;
+      case 'ucs2':
+      case 'ucs-2':
+      case 'utf16le':
+      case 'utf-16le':
+        return len * 2;
+
+      case 'hex':
+        return len >>> 1;
+
+      case 'base64':
+        return base64ByteLength(string, len);
+
+      default:
+        // The C++ binding defaulted to UTF8, we should too.
+        if (loweredCase)
+          return binding.byteLengthUtf8(string);
+
+        encoding = ('' + encoding).toLowerCase();
+        loweredCase = true;
+    }
   }
-
-  return binding.byteLength(string, encoding);
 }
 
 Buffer.byteLength = byteLength;
 
-// toString(encoding, start=0, end=buffer.length)
-Buffer.prototype.toString = function(encoding, start, end) {
+
+// For backwards compatibility.
+Object.defineProperty(Buffer.prototype, 'parent', {
+  enumerable: true,
+  get: function() {
+    if (this.byteLength === 0 ||
+        this.byteLength === this.buffer.byteLength) {
+      return undefined;
+    }
+    return this.buffer;
+  }
+});
+Object.defineProperty(Buffer.prototype, 'offset', {
+  enumerable: true,
+  get: function() {
+    return this.byteOffset;
+  }
+});
+
+
+function slowToString(encoding, start, end) {
   var loweredCase = false;
 
   start = start >>> 0;
@@ -343,6 +346,14 @@ Buffer.prototype.toString = function(encoding, start, end) {
         loweredCase = true;
     }
   }
+}
+
+
+Buffer.prototype.toString = function() {
+  const length = this.length | 0;
+  if (arguments.length === 0)
+    return this.utf8Slice(0, length);
+  return slowToString.apply(this, arguments);
 };
 
 
@@ -423,28 +434,29 @@ Buffer.prototype.fill = function fill(val, start, end) {
 
 
 // XXX remove in v0.13
-Buffer.prototype.get = util.deprecate(function get(offset) {
+Buffer.prototype.get = internalUtil.deprecate(function get(offset) {
   offset = ~~offset;
   if (offset < 0 || offset >= this.length)
     throw new RangeError('index out of range');
   return this[offset];
-}, '.get() is deprecated. Access using array indexes instead.');
+}, 'Buffer.get is deprecated. Use array indexes instead.');
 
 
 // XXX remove in v0.13
-Buffer.prototype.set = util.deprecate(function set(offset, v) {
+Buffer.prototype.set = internalUtil.deprecate(function set(offset, v) {
   offset = ~~offset;
   if (offset < 0 || offset >= this.length)
     throw new RangeError('index out of range');
   return this[offset] = v;
-}, '.set() is deprecated. Set using array indexes instead.');
+}, 'Buffer.set is deprecated. Use array indexes instead.');
 
 
 // TODO(trevnorris): fix these checks to follow new standard
 // write(string, offset = 0, length = buffer.length, encoding = 'utf8')
 var writeWarned = false;
-const writeMsg = '.write(string, encoding, offset, length) is deprecated.' +
-                 ' Use write(string[, offset[, length]][, encoding]) instead.';
+const writeMsg = 'Buffer.write(string, encoding, offset, length) is ' +
+                 'deprecated. Use write(string[, offset[, length]]' +
+                 '[, encoding]) instead.';
 Buffer.prototype.write = function(string, offset, length, encoding) {
   // Buffer#write(string);
   if (offset === undefined) {
@@ -472,16 +484,7 @@ Buffer.prototype.write = function(string, offset, length, encoding) {
 
   // XXX legacy write(string, encoding, offset, length) - remove in v0.13
   } else {
-    if (!writeWarned) {
-      if (process.throwDeprecation)
-        throw new Error(writeMsg);
-      else if (process.traceDeprecation)
-        console.trace(writeMsg);
-      else
-        console.error(writeMsg);
-      writeWarned = true;
-    }
-
+    writeWarned = internalUtil.printDeprecationMessage(writeMsg, writeWarned);
     var swap = encoding;
     encoding = offset;
     offset = length >>> 0;
@@ -544,7 +547,7 @@ Buffer.prototype.toJSON = function() {
 
 // TODO(trevnorris): currently works like Array.prototype.slice(), which
 // doesn't follow the new standard for throwing on out of range indexes.
-Buffer.prototype.slice = function(start, end) {
+Buffer.prototype.slice = function slice(start, end) {
   var len = this.length;
   start = ~~start;
   end = end === undefined ? len : ~~end;
@@ -568,13 +571,7 @@ Buffer.prototype.slice = function(start, end) {
   if (end < start)
     end = start;
 
-  var buf = new NativeBuffer();
-  sliceOnto(this, buf, start, end);
-  buf.length = end - start;
-  if (buf.length > 0)
-    buf.parent = this.parent === undefined ? this : this.parent;
-
-  return buf;
+  return binding.slice(this, start, end);
 };
 
 
@@ -1038,80 +1035,3 @@ Buffer.prototype.writeDoubleBE = function writeDoubleBE(val, offset, noAssert) {
   binding.writeDoubleBE(this, val, offset);
   return offset + 8;
 };
-
-// ES6 iterator
-
-var ITERATOR_KIND_KEYS = 1;
-var ITERATOR_KIND_ENTRIES = 3;
-
-function BufferIteratorResult(value, done) {
-  this.value = value;
-  this.done = done;
-}
-
-var resultCache = new Array(256);
-
-for (var i = 0; i < 256; i++)
-  resultCache[i] = Object.freeze(new BufferIteratorResult(i, false));
-
-var finalResult = Object.freeze(new BufferIteratorResult(undefined, true));
-
-function BufferIterator(buffer, kind) {
-  this._buffer = buffer;
-  this._kind = kind;
-  this._index = 0;
-}
-
-BufferIterator.prototype.next = function() {
-  var buffer = this._buffer;
-  var kind = this._kind;
-  var index = this._index;
-
-  if (index >= buffer.length)
-    return finalResult;
-
-  this._index++;
-
-  if (kind === ITERATOR_KIND_ENTRIES)
-    return new BufferIteratorResult([index, buffer[index]], false);
-
-  return new BufferIteratorResult(index, false);
-};
-
-function BufferValueIterator(buffer) {
-  BufferIterator.call(this, buffer, null);
-}
-
-BufferValueIterator.prototype.next = function() {
-  var buffer = this._buffer;
-  var index = this._index;
-
-  if (index >= buffer.length)
-    return finalResult;
-
-  this._index++;
-
-  return resultCache[buffer[index]];
-};
-
-
-BufferIterator.prototype[Symbol.iterator] = function() {
-  return this;
-};
-
-BufferValueIterator.prototype[Symbol.iterator] =
-    BufferIterator.prototype[Symbol.iterator];
-
-Buffer.prototype.keys = function() {
-  return new BufferIterator(this, ITERATOR_KIND_KEYS);
-};
-
-Buffer.prototype.entries = function() {
-  return new BufferIterator(this, ITERATOR_KIND_ENTRIES);
-};
-
-Buffer.prototype.values = function() {
-  return new BufferValueIterator(this);
-};
-
-Buffer.prototype[Symbol.iterator] = Buffer.prototype.values;
