@@ -10,13 +10,16 @@ try {
   var randomBytes = binding.randomBytes;
   var getCiphers = binding.getCiphers;
   var getHashes = binding.getHashes;
+  var getCurves = binding.getCurves;
 } catch (e) {
   throw new Error('node.js not compiled with openssl crypto support.');
 }
 
+const Buffer = require('buffer').Buffer;
 const constants = require('constants');
 const stream = require('stream');
 const util = require('util');
+const internalUtil = require('internal/util');
 
 const DH_GENERATOR = 2;
 
@@ -278,7 +281,6 @@ Decipheriv.prototype.setAuthTag = Cipher.prototype.setAuthTag;
 Decipheriv.prototype.setAAD = Cipher.prototype.setAAD;
 
 
-
 exports.createSign = exports.Sign = Sign;
 function Sign(algorithm, options) {
   if (!(this instanceof Sign))
@@ -314,7 +316,6 @@ Sign.prototype.sign = function(options, encoding) {
 };
 
 
-
 exports.createVerify = exports.Verify = Verify;
 function Verify(algorithm, options) {
   if (!(this instanceof Verify))
@@ -560,7 +561,6 @@ ECDH.prototype.getPublicKey = function getPublicKey(encoding, format) {
 };
 
 
-
 exports.pbkdf2 = function(password,
                           salt,
                           iterations,
@@ -652,13 +652,17 @@ exports.randomBytes = exports.pseudoRandomBytes = randomBytes;
 exports.rng = exports.prng = randomBytes;
 
 exports.getCiphers = function() {
-  return filterDuplicates(getCiphers.call(null, arguments));
+  return filterDuplicates(getCiphers());
 };
 
 
 exports.getHashes = function() {
-  return filterDuplicates(getHashes.call(null, arguments));
+  return filterDuplicates(getHashes());
+};
 
+
+exports.getCurves = function() {
+  return filterDuplicates(getCurves());
 };
 
 
@@ -679,10 +683,13 @@ function filterDuplicates(names) {
 }
 
 // Legacy API
-exports.__defineGetter__('createCredentials', util.deprecate(function() {
-  return require('tls').createSecureContext;
-}, 'createCredentials() is deprecated, use tls.createSecureContext instead'));
+exports.__defineGetter__('createCredentials',
+  internalUtil.deprecate(function() {
+    return require('tls').createSecureContext;
+  }, 'crypto.createCredentials is deprecated. ' +
+     'Use tls.createSecureContext instead.'));
 
-exports.__defineGetter__('Credentials', util.deprecate(function() {
+exports.__defineGetter__('Credentials', internalUtil.deprecate(function() {
   return require('tls').SecureContext;
-}, 'Credentials is deprecated, use tls.createSecureContext instead'));
+}, 'crypto.Credentials is deprecated. ' +
+   'Use tls.createSecureContext instead.'));
