@@ -4,10 +4,12 @@ const events = require('events');
 const stream = require('stream');
 const timers = require('timers');
 const util = require('util');
+const internalUtil = require('internal/util');
 const assert = require('assert');
 const cares = process.binding('cares_wrap');
 const uv = process.binding('uv');
 
+const Buffer = require('buffer').Buffer;
 const TTYWrap = process.binding('tty_wrap');
 const TCP = process.binding('tcp_wrap').TCP;
 const Pipe = process.binding('pipe_wrap').Pipe;
@@ -91,6 +93,7 @@ function initSocketHandle(self) {
   self.destroyed = false;
   self.bytesRead = 0;
   self._bytesDispatched = 0;
+  self._sockname = null;
 
   // Handle creation may be deferred to bind() or connect() time.
   if (self._handle) {
@@ -268,14 +271,10 @@ function writeAfterFIN(chunk, encoding, cb) {
   // TODO: defer error events consistently everywhere, not just the cb
   self.emit('error', er);
   if (typeof cb === 'function') {
-    process.nextTick(writeAfterFINNT, cb, er);
+    process.nextTick(cb, er);
   }
 }
 
-function writeAfterFINNT(cb, er) {
-  cb(er);
-}
-
 exports.Socket = Socket;
 exports.Stream = Socket; // Legacy naming.
 
@@ -310,6 +309,7 @@ Socket.prototype.setTimeout = function(msecs, callback) {
       this.once('timeout', callback);
     }
   }
+  return this;
 };
 
 
@@ -320,15 +320,30 @@ Socket.prototype._onTimeout = function() {
 
 
 Socket.prototype.setNoDelay = function(enable) {
+  if (!this._handle) {
+    this.once('connect',
+              enable ? this.setNoDelay : this.setNoDelay.bind(this, enable));
+    return this;
+  }
+
   // backwards compatibility: assume true when `enable` is omitted
-  if (this._handle && this._handle.setNoDelay)
+  if (this._handle.setNoDelay)
     this._handle.setNoDelay(enable === undefined ? true : !!enable);
+
+  return this;
 };
 
 
 Socket.prototype.setKeepAlive = function(setting, msecs) {
-  if (this._handle && this._handle.setKeepAlive)
+  if (!this._handle) {
+    this.once('connect', this.setKeepAlive.bind(this, setting, msecs));
+    return this;
+  }
+
+  if (this._handle.setKeepAlive)
     this._handle.setKeepAlive(setting, ~~(msecs / 1000));
+
+  return this;
 };
 
 
@@ -426,9 +441,7 @@ Socket.prototype._destroy = function(exception, cb) {
   function fireErrorCallbacks() {
     if (cb) cb(exception);
     if (exception && !self._writableState.errorEmitted) {
-      process.nextTick(function() {
-        self.emit('error', exception);
-      });
+      process.nextTick(emitErrorNT, self, exception);
       self._writableState.errorEmitted = true;
     }
   }
@@ -457,6 +470,7 @@ Socket.prototype._destroy = function(exception, cb) {
     });
     this._handle.onread = noop;
     this._handle = null;
+    this._sockname = null;
   }
 
   // we set destroyed to true before firing error callbacks in order
@@ -859,6 +873,7 @@ Socket.prototype.connect = function(options, cb) {
     this.destroyed = false;
     this._handle = null;
     this._peername = null;
+    this._sockname = null;
   }
 
   var self = this;
@@ -914,7 +929,10 @@ function lookupAndConnect(self, options) {
   // TODO(evanlucas) should we hot path this for localhost?
   var addressType = exports.isIP(host);
   if (addressType) {
-    connect(self, host, port, addressType, localAddress, localPort);
+    process.nextTick(function() {
+      if (self._connecting)
+        connect(self, host, port, addressType, localAddress, localPort);
+    });
     return;
   }
 
@@ -926,11 +944,19 @@ function lookupAndConnect(self, options) {
     hints: 0
   };
 
-  if (dnsopts.family !== 4 && dnsopts.family !== 6)
-    dnsopts.hints = dns.ADDRCONFIG | dns.V4MAPPED;
+  if (dnsopts.family !== 4 && dnsopts.family !== 6) {
+    dnsopts.hints = dns.ADDRCONFIG;
+    // The AI_V4MAPPED hint is not supported on FreeBSD, and getaddrinfo
+    // returns EAI_BADFLAGS. However, it seems to be supported on most other
+    // systems. See
+    // http://lists.freebsd.org/pipermail/freebsd-bugs/2008-February/028260.html
+    // for more information on the lack of support for FreeBSD.
+    if (process.platform !== 'freebsd')
+      dnsopts.hints |= dns.V4MAPPED;
+  }
 
   debug('connect: find host ' + host);
-  debug('connect: dns options ' + dnsopts);
+  debug('connect: dns options', dnsopts);
   self._host = host;
   var lookup = options.lookup || dns.lookup;
   lookup(host, dnsopts, function(err, ip, addressType) {
@@ -946,7 +972,10 @@ function lookupAndConnect(self, options) {
       // immediately calls net.Socket.connect() on it (that's us).
       // There are no event listeners registered yet so defer the
       // error event to the next tick.
-      process.nextTick(connectErrorNT, self, err, options);
+      err.host = options.host;
+      err.port = options.port;
+      err.message = err.message + ' ' + options.host + ':' + options.port;
+      process.nextTick(connectErrorNT, self, err);
     } else {
       self._unrefTimer();
       connect(self,
@@ -960,26 +989,34 @@ function lookupAndConnect(self, options) {
 }
 
 
-function connectErrorNT(self, err, options) {
-  err.host = options.host;
-  err.port = options.port;
-  err.message = err.message + ' ' + options.host + ':' + options.port;
+function connectErrorNT(self, err) {
   self.emit('error', err);
   self._destroy();
 }
 
 
 Socket.prototype.ref = function() {
-  if (this._handle)
-    this._handle.ref();
+  if (!this._handle) {
+    this.once('connect', this.ref);
+    return this;
+  }
+
+  this._handle.ref();
+
+  return this;
 };
 
 
 Socket.prototype.unref = function() {
-  if (this._handle)
-    this._handle.unref();
-};
+  if (!this._handle) {
+    this.once('connect', this.unref);
+    return this;
+  }
 
+  this._handle.unref();
+
+  return this;
+};
 
 
 function afterConnect(status, handle, req, readable, writable) {
@@ -998,6 +1035,7 @@ function afterConnect(status, handle, req, readable, writable) {
 
   assert.ok(self._connecting);
   self._connecting = false;
+  self._sockname = null;
 
   if (status == 0) {
     self.readable = readable;
@@ -1053,16 +1091,17 @@ function Server(options, connectionListener) {
   this._connections = 0;
 
   Object.defineProperty(this, 'connections', {
-    get: util.deprecate(function() {
+    get: internalUtil.deprecate(function() {
 
       if (self._usingSlaves) {
         return null;
       }
       return self._connections;
-    }, 'connections property is deprecated. Use getConnections() method'),
-    set: util.deprecate(function(val) {
+    }, 'Server.connections property is deprecated. ' +
+       'Use Server.getConnections method instead.'),
+    set: internalUtil.deprecate(function(val) {
       return (self._connections = val);
-    }, 'connections property is deprecated. Use getConnections() method'),
+    }, 'Server.connections property is deprecated.'),
     configurable: true, enumerable: false
   });
 
@@ -1177,9 +1216,7 @@ Server.prototype._listen2 = function(address, port, addressType, backlog, fd) {
 
     if (typeof rval === 'number') {
       var error = exceptionWithHostPort(rval, 'listen', address, port);
-      process.nextTick(function() {
-        self.emit('error', error);
-      });
+      process.nextTick(emitErrorNT, self, error);
       return;
     }
     self._handle = rval;
@@ -1194,9 +1231,7 @@ Server.prototype._listen2 = function(address, port, addressType, backlog, fd) {
     var ex = exceptionWithHostPort(err, 'listen', address, port);
     self._handle.close();
     self._handle = null;
-    process.nextTick(function() {
-      self.emit('error', ex);
-    });
+    process.nextTick(emitErrorNT, self, ex);
     return;
   }
 
@@ -1211,6 +1246,11 @@ Server.prototype._listen2 = function(address, port, addressType, backlog, fd) {
 };
 
 
+function emitErrorNT(self, err) {
+  self.emit('error', err);
+}
+
+
 function emitListeningNT(self) {
   // ensure handle hasn't closed
   if (self._handle)
@@ -1384,9 +1424,7 @@ function onconnection(err, clientHandle) {
 
 Server.prototype.getConnections = function(cb) {
   function end(err, connections) {
-    process.nextTick(function() {
-      cb(err, connections);
-    });
+    process.nextTick(cb, err, connections);
   }
 
   if (!this._usingSlaves) {
@@ -1465,16 +1503,19 @@ Server.prototype._emitCloseIfDrained = function() {
     return;
   }
 
-  process.nextTick(function() {
-    debug('SERVER: emit close');
-    self.emit('close');
-  });
+  process.nextTick(emitCloseNT, self);
 };
 
 
-Server.prototype.listenFD = util.deprecate(function(fd, type) {
+function emitCloseNT(self) {
+  debug('SERVER: emit close');
+  self.emit('close');
+}
+
+
+Server.prototype.listenFD = internalUtil.deprecate(function(fd, type) {
   return this.listen({ fd: fd });
-}, 'listenFD is deprecated. Use listen({fd: <number>}).');
+}, 'Server.listenFD is deprecated. Use Server.listen({fd: <number>}) instead.');
 
 Server.prototype._setupSlave = function(socketList) {
   this._usingSlaves = true;
@@ -1486,6 +1527,8 @@ Server.prototype.ref = function() {
 
   if (this._handle)
     this._handle.ref();
+
+  return this;
 };
 
 Server.prototype.unref = function() {
@@ -1493,6 +1536,8 @@ Server.prototype.unref = function() {
 
   if (this._handle)
     this._handle.unref();
+
+  return this;
 };
 
 
