@@ -61,8 +61,7 @@ exports.writer = util.inspect;
 exports._builtinLibs = ['assert', 'buffer', 'child_process', 'cluster',
   'crypto', 'dgram', 'dns', 'domain', 'events', 'fs', 'http', 'https', 'net',
   'os', 'path', 'punycode', 'querystring', 'readline', 'stream',
-  'string_decoder', 'tls', 'tty', 'url', 'util', 'v8', 'vm', 'zlib',
-  'smalloc'];
+  'string_decoder', 'tls', 'tty', 'url', 'util', 'v8', 'vm', 'zlib'];
 
 
 const BLOCK_SCOPED_ERROR = 'Block-scoped declarations (let, ' +
@@ -116,6 +115,12 @@ function REPLServer(prompt,
   // just for backwards compat, see github.com/joyent/node/pull/7127
   self.rli = this;
 
+  const savedRegExMatches = ['', '', '', '', '', '', '', '', '', ''];
+  const sep = '\u0000\u0000\u0000';
+  const regExMatcher = new RegExp(`^${sep}(.*)${sep}(.*)${sep}(.*)${sep}(.*)` +
+                                  `${sep}(.*)${sep}(.*)${sep}(.*)${sep}(.*)` +
+                                  `${sep}(.*)$`);
+
   eval_ = eval_ || defaultEval;
 
   function defaultEval(code, context, file, cb) {
@@ -149,6 +154,10 @@ function REPLServer(prompt,
       break;
     }
 
+    // This will set the values from `savedRegExMatches` to corresponding
+    // predefined RegExp properties `RegExp.$1`, `RegExp.$2` ... `RegExp.$9`
+    regExMatcher.test(savedRegExMatches.join(sep));
+
     if (!err) {
       try {
         if (self.useGlobal) {
@@ -167,6 +176,12 @@ function REPLServer(prompt,
       }
     }
 
+    // After executing the current expression, store the values of RegExp
+    // predefined properties back in `savedRegExMatches`
+    for (let idx = 1; idx < savedRegExMatches.length; idx += 1) {
+      savedRegExMatches[idx] = RegExp[`$${idx}`];
+    }
+
     cb(err, result);
   }
 
@@ -175,6 +190,7 @@ function REPLServer(prompt,
   self._domain.on('error', function(e) {
     debug('domain error');
     self.outputStream.write((e.stack || e) + '\n');
+    self._currentStringLiteral = null;
     self.bufferedCommand = '';
     self.lines.level = [];
     self.displayPrompt();
@@ -201,6 +217,8 @@ function REPLServer(prompt,
   self.outputStream = output;
 
   self.resetContext();
+  // Initialize the current string literal found, to be null
+  self._currentStringLiteral = null;
   self.bufferedCommand = '';
   self.lines.level = [];
 
@@ -218,7 +236,7 @@ function REPLServer(prompt,
 
   self.setPrompt(prompt !== undefined ? prompt : '> ');
 
-  this.commands = {};
+  this.commands = Object.create(null);
   defineDefaultCommands(this);
 
   // figure out which "writer" function to use
@@ -259,21 +277,86 @@ function REPLServer(prompt,
       sawSIGINT = false;
     }
 
+    self._currentStringLiteral = null;
     self.bufferedCommand = '';
     self.lines.level = [];
     self.displayPrompt();
   });
 
+  function parseLine(line, currentStringLiteral) {
+    var previous = null, current = null;
+
+    for (var i = 0; i < line.length; i += 1) {
+      if (previous === '\\') {
+        // if it is a valid escaping, then skip processing
+        previous = current;
+        continue;
+      }
+
+      current = line.charAt(i);
+      if (current === currentStringLiteral) {
+        currentStringLiteral = null;
+      } else if (current === '\'' ||
+                 current === '"' &&
+                 currentStringLiteral === null) {
+        currentStringLiteral = current;
+      }
+      previous = current;
+    }
+
+    return currentStringLiteral;
+  }
+
+  function getFinisherFunction(cmd, defaultFn) {
+    if ((self._currentStringLiteral === null &&
+         cmd.charAt(cmd.length - 1) === '\\') ||
+        (self._currentStringLiteral !== null &&
+         cmd.charAt(cmd.length - 1) !== '\\')) {
+
+      // If the line continuation is used outside string literal or if the
+      // string continuation happens with out line continuation, then fail hard.
+      // Even if the error is recoverable, get the underlying error and use it.
+      return function(e, ret) {
+        var error = e instanceof Recoverable ? e.err : e;
+
+        if (arguments.length === 2) {
+          // using second argument only if it is actually passed. Otherwise
+          // `undefined` will be printed when invalid REPL commands are used.
+          return defaultFn(error, ret);
+        }
+
+        return defaultFn(error);
+      };
+    }
+    return defaultFn;
+  }
+
   self.on('line', function(cmd) {
     debug('line %j', cmd);
     sawSIGINT = false;
     var skipCatchall = false;
+    var finisherFn = finish;
 
     // leading whitespaces in template literals should not be trimmed.
     if (self._inTemplateLiteral) {
       self._inTemplateLiteral = false;
     } else {
-      cmd = trimWhitespace(cmd);
+      const wasWithinStrLiteral = self._currentStringLiteral !== null;
+      self._currentStringLiteral = parseLine(cmd, self._currentStringLiteral);
+      const isWithinStrLiteral = self._currentStringLiteral !== null;
+
+      if (!wasWithinStrLiteral && !isWithinStrLiteral) {
+        // Current line has nothing to do with String literals, trim both ends
+        cmd = cmd.trim();
+      } else if (wasWithinStrLiteral && !isWithinStrLiteral) {
+        // was part of a string literal, but it is over now, trim only the end
+        cmd = cmd.trimRight();
+      } else if (isWithinStrLiteral && !wasWithinStrLiteral) {
+        // was not part of a string literal, but it is now, trim only the start
+        cmd = cmd.trimLeft();
+      }
+
+      finisherFn = getFinisherFunction(cmd, finish);
     }
 
     // Check to see if a REPL keyword was used. If it returns true,
@@ -290,7 +373,7 @@ function REPLServer(prompt,
       }
     }
 
-    if (!skipCatchall) {
+    if (!skipCatchall && (cmd || (!cmd && self.bufferedCommand))) {
       var evalCmd = self.bufferedCommand + cmd;
       if (/^\s*\{/.test(evalCmd) && /\}\s*$/.test(evalCmd)) {
         // It's confusing for `{ a : 1 }` to be interpreted as a block
@@ -306,9 +389,9 @@ function REPLServer(prompt,
       }
 
       debug('eval %j', evalCmd);
-      self.eval(evalCmd, self.context, 'repl', finish);
+      self.eval(evalCmd, self.context, 'repl', finisherFn);
     } else {
-      finish(null);
+      finisherFn(null);
     }
 
     function finish(e, ret) {
@@ -319,6 +402,7 @@ function REPLServer(prompt,
         self.outputStream.write('npm should be run outside of the ' +
                                 'node repl, in your normal shell.\n' +
                                 '(Press Control-D to exit.)\n');
+        self._currentStringLiteral = null;
         self.bufferedCommand = '';
         self.displayPrompt();
         return;
@@ -340,10 +424,16 @@ function REPLServer(prompt,
       }
 
       // Clear buffer if no SyntaxErrors
+      self._currentStringLiteral = null;
       self.bufferedCommand = '';
 
       // If we got any output - print it (if no error)
-      if (!e && (!self.ignoreUndefined || ret !== undefined)) {
+      if (!e &&
+          // When an invalid REPL command is used, error message is printed
+          // immediately. We don't have to print anything else. So, only when
+          // the second argument to this function is there, print it.
+          arguments.length === 2 &&
+          (!self.ignoreUndefined || ret !== undefined)) {
         self.context._ = ret;
         self.outputStream.write(self.writer(ret) + '\n');
       }
@@ -471,6 +561,15 @@ const requireRE = /\brequire\s*\(['"](([\w\.\/-]+\/)?([\w\.\/-]*))/;
 const simpleExpressionRE =
     /(([a-zA-Z_$](?:\w|\$)*)\.)*([a-zA-Z_$](?:\w|\$)*)\.?$/;
 
+function intFilter(item) {
+  // filters out anything not starting with A-Z, a-z, $ or _
+  return /^[A-Za-z_$]/.test(item);
+}
+
+function filteredOwnPropertyNames(obj) {
+  if (!obj) return [];
+  return Object.getOwnPropertyNames(obj).filter(intFilter);
+}
 
 // Provide a list of completions for the given leading text. This is
 // given to the readline interface for handling tab completion.
@@ -612,19 +711,17 @@ REPLServer.prototype.complete = function(line, callback) {
       if (!expr) {
         // If context is instance of vm.ScriptContext
         // Get global vars synchronously
-        if (this.useGlobal ||
-            this.context.constructor &&
-            this.context.constructor.name === 'Context') {
+        if (this.useGlobal || vm.isContext(this.context)) {
           var contextProto = this.context;
           while (contextProto = Object.getPrototypeOf(contextProto)) {
-            completionGroups.push(Object.getOwnPropertyNames(contextProto));
+            completionGroups.push(filteredOwnPropertyNames(contextProto));
           }
-          completionGroups.push(Object.getOwnPropertyNames(this.context));
+          completionGroups.push(filteredOwnPropertyNames(this.context));
           addStandardGlobals(completionGroups, filter);
           completionGroupsLoaded();
         } else {
           this.eval('.scope', this.context, 'repl', function(err, globals) {
-            if (err || !globals) {
+            if (err || !Array.isArray(globals)) {
               addStandardGlobals(completionGroups, filter);
             } else if (Array.isArray(globals[0])) {
               // Add grouped globals
@@ -644,7 +741,14 @@ REPLServer.prototype.complete = function(line, callback) {
 
           if (obj != null) {
             if (typeof obj === 'object' || typeof obj === 'function') {
-              memberGroups.push(Object.getOwnPropertyNames(obj));
+              try {
+                memberGroups.push(filteredOwnPropertyNames(obj));
+              } catch (ex) {
+                // Probably a Proxy object without `getOwnPropertyNames` trap.
+                // We simply ignore it here, as we don't want to break the
+                // autocompletion. Fixes the bug
+                // https://github.com/nodejs/io.js/issues/2119
+              }
             }
             // works for non-objects
             try {
@@ -656,7 +760,7 @@ REPLServer.prototype.complete = function(line, callback) {
                 p = obj.constructor ? obj.constructor.prototype : null;
               }
               while (p !== null) {
-                memberGroups.push(Object.getOwnPropertyNames(p));
+                memberGroups.push(filteredOwnPropertyNames(p));
                 p = Object.getPrototypeOf(p);
                 // Circular refs possible? Let's guard against that.
                 sentinel--;
@@ -858,10 +962,10 @@ function addStandardGlobals(completionGroups, filter) {
 }
 
 function defineDefaultCommands(repl) {
-  // TODO remove me after 0.3.x
   repl.defineCommand('break', {
     help: 'Sometimes you get stuck, this gets you out',
     action: function() {
+      this._currentStringLiteral = null;
       this.bufferedCommand = '';
       this.displayPrompt();
     }
@@ -876,6 +980,7 @@ function defineDefaultCommands(repl) {
   repl.defineCommand('clear', {
     help: clearMessage,
     action: function() {
+      this._currentStringLiteral = null;
       this.bufferedCommand = '';
       if (!this.useGlobal) {
         this.outputStream.write('Clearing context...\n');
@@ -941,18 +1046,6 @@ function defineDefaultCommands(repl) {
   });
 }
 
-
-function trimWhitespace(cmd) {
-  const trimmer = /^\s*(.+)\s*$/m;
-  var matches = trimmer.exec(cmd);
-
-  if (matches && matches.length === 2) {
-    return matches[1];
-  }
-  return '';
-}
-
-
 function regexpEscape(s) {
   return s.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, '\\$&');
 }
@@ -997,7 +1090,7 @@ function isRecoverableError(e, self) {
       self._inTemplateLiteral = true;
       return true;
     }
-    return /^(Unexpected end of input|Unexpected token :)/.test(message);
+    return /^(Unexpected end of input|Unexpected token)/.test(message);
   }
   return false;
 }
