@@ -1,7 +1,8 @@
+'use strict';
 var assert = require('assert');
 var child_process = require('child_process');
 var spawn = child_process.spawn;
-var cmd = (process.platform === 'win32') ? 'rundll32' : 'ls';
+var cmd = require('../common').isWindows ? 'rundll32' : 'ls';
 var invalidArgsMsg = /Incorrect value of args option/;
 var invalidOptionsMsg = /options argument must be an object/;
 
