@@ -1,5 +1,6 @@
+'use strict';
 var assert = require('assert');
-var cluster = require('cluster')
+var cluster = require('cluster');
 var net = require('net');
 
 var SENTINEL = 42;
