@@ -1,4 +1,4 @@
-// USE OR OTHER DEALINGS IN THE SOFTWARE.
+'use strict';
 
 var common = require('../common');
 var assert = require('assert');
@@ -7,7 +7,7 @@ var events = require('events');
 var EventEmitter = require('events').EventEmitter;
 var assert = require('assert');
 
-var e = new EventEmitter;
+var e = new EventEmitter();
 var fl;  // foo listeners
 
 fl = e.listeners('foo');
