@@ -1,3 +1,4 @@
+'use strict';
 var common = require('../common');
 var fs = require('fs');
 var path = require('path');
@@ -10,12 +11,7 @@ var fileNameLen = Math.max(260 - common.tmpDir.length - 1, 1);
 var fileName = path.join(common.tmpDir, new Array(fileNameLen + 1).join('x'));
 var fullPath = path.resolve(fileName);
 
-try {
-  fs.unlinkSync(fullPath);
-}
-catch (e) {
-  // Ignore.
-}
+common.refreshTmpDir();
 
 console.log({
   filenameLength: fileName.length,
