@@ -1,13 +1,16 @@
+'use strict';
 var common = require('../common');
 var assert = require('assert');
 var path = require('path');
-var childProccess = require('child_process');
+var childProcess = require('child_process');
 var fs = require('fs');
 
 var scriptString = path.join(common.fixturesDir, 'print-chars.js');
 var scriptBuffer = path.join(common.fixturesDir, 'print-chars-from-buffer.js');
 var tmpFile = path.join(common.tmpDir, 'stdout.txt');
 
+common.refreshTmpDir();
+
 function test(size, useBuffer, cb) {
   var cmd = '"' + process.argv[0] + '"' +
             ' ' +
@@ -23,7 +26,7 @@ function test(size, useBuffer, cb) {
 
   common.print(size + ' chars to ' + tmpFile + '...');
 
-  childProccess.exec(cmd, function(err) {
+  childProcess.exec(cmd, function(err) {
     if (err) throw err;
 
     console.log('done!');
