@@ -5,18 +5,17 @@ set -e
 # always change the working directory to the project's root directory
 cd $(dirname $0)/..
 
-# pass NODE_EXE from something like Makefile
-# it should point to either {./}node or {./}node.exe, depending on the platform
-if [ -z $NODE_EXE ]; then
+# pass a $NODE environment variable from something like Makefile
+# it should point to either ./iojs or ./iojs.exe, depending on the platform
+if [ -z $NODE ]; then
   echo "No node executable provided. Bailing." >&2
   exit 0
 fi
 
 rm -rf test-npm
-mkdir test-npm
 
 # make a copy of deps/npm to run the tests on
-cp -r deps/npm/ test-npm/
+cp -r deps/npm test-npm
 
 cd test-npm
 
@@ -30,8 +29,10 @@ export npm_config_prefix="npm-prefix"
 export npm_config_tmp="npm-tmp"
 
 # install npm devDependencies and run npm's tests
-../$NODE_EXE cli.js install --ignore-scripts
-../$NODE_EXE cli.js run-script test-all
+
+../$NODE cli.js install --ignore-scripts
+../$NODE cli.js run-script test-legacy
+../$NODE cli.js run-script test
 
 # clean up everything one single shot
 cd .. && rm -rf test-npm
